@@ -3,6 +3,7 @@
 const Cinnamon = imports.gi.Cinnamon;
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
+const Gtk = imports.gi.Gtk;
 const Lang = imports.lang;
 const Signals = imports.signals;
 const St = imports.gi.St;
@@ -11,6 +12,7 @@ const AppletManager = imports.ui.appletManager;
 const Config = imports.misc.config;
 const DeskletManager = imports.ui.deskletManager;
 const ExtensionSystem = imports.ui.extensionSystem;
+const SearchProviderManager = imports.ui.searchProviderManager;
 const Main = imports.ui.main;
 
 const State = {
@@ -20,14 +22,47 @@ const State = {
     OUT_OF_DATE: 3
 };
 
-// Maps uuid -> importer object (extension directory tree)
-const importObjects = {};
+// macro for creating extension types
+function _createExtensionType(name, folder, manager, overrides){
+    let type = {
+        name: name,
+        folder: folder,
+        requiredFunctions: ["main"],
+        requiredProperties: ["uuid", "name", "description"],
+        niceToHaveProperties: [],
+        roles: {},
+        callbacks: {
+            finishExtensionLoad: manager.finishExtensionLoad,
+            prepareExtensionUnload: manager.prepareExtensionUnload
+        },
+        maps: {
+            importObjects: {},
+            objects: {},
+            meta: {},
+            dirs: {}
+        }
+    };
 
-// Maps uuid -> Extension object
-const objects = {};
+    for(let prop in overrides)
+        type[prop] = overrides[prop];
 
-// Maps uuid -> metadata object
-const meta = {};
+    // Add signal methods
+    Signals.addSignalMethods(type);
+
+    let path = GLib.build_filenamev([global.userdatadir, folder]);
+    type.userDir = path;
+
+    // create user directories if they don't exist.
+    let dir = Gio.file_new_for_path(type.userDir)
+    try {
+        if(!dir.query_exists(null))
+            dir.make_directory_with_parents(null);
+    } catch(e){
+        global.logError(e);
+    }
+
+    return type;
+}
 
 /**
  * const Type:
@@ -48,106 +83,76 @@ const meta = {};
  * Properties are nested, with lowerCamelCase properties (e.g. requiredFunctions) as sub-properties of CAPITAL one (EXTENSION). Thus they are refered to as, e.g., Type.EXTENSION.requiredFunctions
  */
 const Type = {
-    EXTENSION: {
-        name: 'Extension',
-        folder: 'extensions',
-        requiredFunctions: ['init', 'disable', 'enable'],
-        requiredProperties: ['uuid', 'name', 'description', 'cinnamon-version'],
-        niceToHaveProperties: ['url'],
-        roles: {},
-        callbacks: {
-            finishExtensionLoad: ExtensionSystem.finishExtensionLoad,
-            prepareExtensionUnload: ExtensionSystem.prepareExtensionUnload
+    EXTENSION: _createExtensionType("Extension", "extensions", ExtensionSystem, {
+        requiredFunctions: ["init", "disable", "enable"],
+        requiredProperties: ["uuid", "name", "description", "cinnamon-version"],
+        niceToHaveProperties: ["url"],
+    }),
+    APPLET: _createExtensionType("Applet", "applets", AppletManager, {
+        roles: {
+            notifications: null,
+            windowlist: null,
+            panellauncher: null
         }
-    },
-    APPLET: {
-        name: 'Applet',
-        folder: 'applets',
-        requiredFunctions: ['main'],
-        requiredProperties: ['uuid', 'name', 'description'],
-        niceToHaveProperties: [],
+    }),
+    DESKLET: _createExtensionType("Desklet", "desklets", DeskletManager, {
         roles: {
             notifications: null,
             windowlist: null
-        },
-        callbacks: {
-            finishExtensionLoad: AppletManager.finishExtensionLoad,
-            prepareExtensionUnload: AppletManager.prepareExtensionUnload
         }
-    },
-    DESKLET: {
-        name: 'Desklet',
-        folder: 'desklets',
-        requiredFunctions: ['main'],
-        requiredProperties: ['uuid', 'name', 'description'],
-        niceToHaveProperties: [],
-        roles: {
-            notifications: null,
-            windowlist: null
-        },
-        callbacks: {
-            finishExtensionLoad: DeskletManager.finishExtensionLoad,
-            prepareExtensionUnload: DeskletManager.prepareExtensionUnload
-        }
-    }
+    }),
+    SEARCH_PROVIDER: _createExtensionType("Search provider", "search_providers", SearchProviderManager, {
+        requiredFunctions: ["perform_search", "on_result_selected"]
+    })
 };
 
-// Add signal methods to all types and create user directories if they don't exist.
-for(var key in Type) {
-    let type = Type[key];
-    Signals.addSignalMethods(type);
-
-    let path = GLib.build_filenamev([global.userdatadir, type.folder]);
-    type.userDir = Gio.file_new_for_path(path);
-    try {
-        if (!type.userDir.query_exists(null))
-            type.userDir.make_directory_with_parents(null);
-    } catch (e) {
-        global.logError(e);
-    }
-}
-
 // Create a dummy metadata object when metadata parsing failed or was not done yet.
-function createMetaDummy(uuid, path, state) {
-    return { name: uuid, description: 'Metadata load failed', state: state, path: path, error: '' };
+function createMetaDummy(uuid, path, state, type) {
+    return { name: uuid, description: 'Metadata load failed', state: state, path: path, error: '', type: type };
 }
 
 // The Extension object itself
-function Extension(dir, type) {
-    this._init(dir, type);
+function Extension(dir, type, force, uuid) {
+    this._init(dir, type, force, uuid);
 }
 
 Extension.prototype = {
-    _init: function(dir, type) {
-        this.uuid = dir.get_basename();
+    _init: function(dir, type, force, uuid) {
+        this.uuid = uuid;
         this.dir = dir;
         this.type = type;
-        this.lowerType = type.name.toLowerCase();
+        this.lowerType = type.name.toLowerCase().replace(" ", "_");
         this.theme = null;
         this.stylesheet = null;
-        this.meta = createMetaDummy(this.uuid, dir.get_path(), State.INITIALIZING);
+        this.iconDirectory = null;
+        this.meta = createMetaDummy(this.uuid, dir.get_path(), State.INITIALIZING, type);
         this.startTime = new Date().getTime();
 
         this.loadMetaData(dir.get_child('metadata.json'));
-        this.validateMetaData();
+        if (!force)
+            this.validateMetaData();
 
-        this.ensureFileExists(dir.get_child(this.lowerType + '.js'));
-        this.loadStylesheet(dir.get_child('stylesheet.css'));
+        if (this.meta.multiversion) {
+            this.dir = findExtensionSubdirectory(this.dir);
+            this.meta.path = this.dir.get_path();
+            type.maps.dirs[this.uuid] = this.dir;
+        }
+
+        this.ensureFileExists(this.dir.get_child(this.lowerType + '.js'));
+        this.loadStylesheet(this.dir.get_child('stylesheet.css'));
         
         if (this.stylesheet) {
             Main.themeManager.connect('theme-set', Lang.bind(this, function() {
                 this.loadStylesheet(this.dir.get_child('stylesheet.css'));
             }));
         }
+        this.loadIconDirectory(this.dir);
+
+        imports.addSubImporter(this.lowerType, this.meta.path);
+        type.maps.importObjects[this.uuid] = imports[this.lowerType];
 
         try {
-            global.add_extension_importer('imports.ui.extension.importObjects', this.uuid, this.meta.path);
-        } catch (e) {
-            throw this.logError('Error importing extension ' + this.uuid + ' from path ' + this.meta.path, e);
-        }
-
-        try {
-            this.module = importObjects[this.uuid][this.lowerType]; // get [extension/applet/desklet].js
+            this.module = type.maps.importObjects[this.uuid][this.lowerType]; // get [extension/applet/desklet].js
         } catch (e) {
             throw this.logError('Error importing ' + this.lowerType + '.js from ' + this.uuid, e);
         }
@@ -159,7 +164,7 @@ Extension.prototype = {
             }
         }
 
-        objects[this.uuid] = this;
+        type.maps.objects[this.uuid] = this;
     },
 
     finalize : function() {
@@ -191,6 +196,7 @@ Extension.prototype = {
         if(this.meta.state == State.INITIALIZING) {
             this.unlockRole();
             this.unloadStylesheet();
+            this.unloadIconDirectory();
             forgetExtension(this.uuid);
         }
         error._alreadyLogged = true;
@@ -202,23 +208,25 @@ Extension.prototype = {
     },
 
     loadMetaData: function(metadataFile) {
-        this.ensureFileExists(metadataFile);
 
         let oldState = this.meta.state;
         let oldPath = this.meta.path;
 
         try {
+            this.ensureFileExists(metadataFile);
             let metadataContents = Cinnamon.get_file_contents_utf8_sync(metadataFile.get_path());
             this.meta = JSON.parse(metadataContents);
             
             // Store some additional crap here
             this.meta.state = oldState;
             this.meta.path = oldPath;
+            this.meta.type = this.type;
+            this.meta.error = '';
 
-            meta[this.uuid] = this.meta;
+            this.type.maps.meta[this.uuid] = this.meta;
         } catch (e) {
-            this.meta = createMetaDummy(this.uuid, oldPath, oldState);
-            meta[this.uuid] = this.meta;
+            this.meta = createMetaDummy(this.uuid, oldPath, State.ERROR, this.type);
+            this.type.maps.meta[this.uuid] = this.meta;
             throw this.logError('Failed to load/parse metadata.json', e);
         }
     },
@@ -248,12 +256,6 @@ Extension.prototype = {
             if (!(role in this.type.roles)) {
                 throw this.logError('Unknown role definition: ' + role + ' in metadata.json');
             }
-            
-            let maxInstances;
-            try { maxInstances = parseInt(this.meta['max-instances']); } catch(e) { maxInstances = 1; }
-            if(maxInstances > 1) {
-                throw this.logError(this.type.name + 's with a role can only have one instance. The metadata.json suggests otherwise (max-instances > 1)');
-            }
         }
     },
 
@@ -297,6 +299,29 @@ Extension.prototype = {
             }
         }
     },
+    
+    loadIconDirectory: function(dir) {
+        let iconDir = dir.get_child("icons");
+        if (iconDir.query_exists(null)) {
+            let path = iconDir.get_path();
+            this.iconDirectory = path;
+            Gtk.IconTheme.get_default().append_search_path(path);
+        }
+    },
+    
+    unloadIconDirectory: function() {
+        if (this.iconDirectory) {
+            let iconTheme = Gtk.IconTheme.get_default();
+            let searchPath = iconTheme.get_search_path();
+            for (let i = 0; i < searchPath.length; i++) {
+                if (searchPath[i] == this.iconDirectory) {
+                    searchPath.splice(i,1);
+                    iconTheme.set_search_path(searchPath);
+                    break;
+                }
+            }
+        }
+    },
 
     ensureFileExists: function(file) {
         if (!file.query_exists(null)) {
@@ -308,7 +333,6 @@ Extension.prototype = {
         let role = this.meta['role'];
         if(role && this.type.roles[role] != this) {
             if(this.type.roles[role] != null) {
-                this.logError('Role ' + role + ' already taken by ' + this.lowerType + ': ' + this.type.roles[role].uuid);
                 return false;
             }
         
@@ -316,6 +340,7 @@ Extension.prototype = {
                 this.type.roles[role] = this;
                 this.roleProvider = roleProvider;
                 global.log("Role locked: " + role);
+                return true;
             }
         }
 
@@ -360,6 +385,34 @@ function versionCheck(required, current) {
     return false;
 }
 
+/**
+ * versionLeq:
+ * @a (string): the first version
+ * @b (string): the second version
+ *
+ * Returns: whether a <= b
+ */
+function versionLeq(a, b) {
+    a = a.split('.');
+    b = b.split('.');
+
+    if (a.length == 2)
+        a.push(0);
+
+    if (b.length == 2)
+        b.push(0);
+
+    for (let i = 0; i < 3; i++) {
+        if (a[i] == b[i])
+            continue;
+        else if (a[i] > b[i])
+            return false;
+        else
+            return true;
+    }
+    return true;
+}
+
 // Returns a string version of a State value
 function getMetaStateString(state) {
     switch (state) {
@@ -375,36 +428,66 @@ function getMetaStateString(state) {
     return 'Unknown'; // Not translated, shouldn't appear
 }
 
+/**
+ * loadExtension:
+ *
+ * @uuid (string): uuid of xlet
+ * @type (Extension.Type): type of xlet
+ */
 function loadExtension(uuid, type) {
-    let extension = objects[uuid];
+    let force = uuid.indexOf("!") == 0;
+    uuid = uuid.replace(/^!/,'');
+
+    let extension = type.maps.objects[uuid];
     if(!extension) {
-        var forgetMeta = true;
         try {
-            let dir = findExtensionDirectory(uuid, type);
-            if(dir == null) {
-                global.logError(type.name + ' ' + uuid + ' not found.');
-                return null;
-            }
-            extension = new Extension(dir, type);
-            forgetMeta = false;
+            type.maps.dirs[uuid] = findExtensionDirectory(uuid, type);
+
+            if (type.maps.dirs[uuid] == null)
+                throw ("not-found");
+
+            extension = new Extension(type.maps.dirs[uuid], type, force, uuid);
 
             if(!type.callbacks.finishExtensionLoad(extension))
-                return null;
+                throw (type.name + ' ' + uuid + ': Could not create applet object.');
 
             extension.finalize();
-        } catch(e) {
-            forgetExtension(uuid, forgetMeta);
+            Main.cinnamonDBusService.EmitXletAddedComplete(true, uuid);
+        } catch (e) {
+            /* Silently fail to load xlets that aren't actually installed - 
+               but no error, since the user can't do anything about it anyhow
+               (short of editing gsettings).  Silent failure is consistent with
+               other reactions in Cinnamon to missing items (e.g. panel launchers
+               just don't show up if their program isn't installed, but we don't
+               remove them or anything) */
+            if (e == "not-found") {
+                forgetExtension(uuid, type, true);
+                return null;
+            }
+            Main.cinnamonDBusService.EmitXletAddedComplete(false, uuid);
+            Main.xlet_startup_error = true;
+            forgetExtension(uuid, type);
             if(e._alreadyLogged)
                 e = undefined;
             global.logError('Could not load ' + type.name.toLowerCase() + ' ' + uuid, e);
+            type.maps.meta[uuid].state = State.ERROR;
+            if (!type.maps.meta[uuid].name)
+                type.maps.meta[uuid].name = uuid
             return null;
         }
     }
     return extension;
 }
 
-function unloadExtension(uuid) {
-    let extension = objects[uuid];
+/**
+ * unloadExtension:
+ *
+ * @uuid (string): uuid of xlet
+ * @type (Extension.Type): type of xlet
+ * @deleteConfig (bool): delete also config files, defaults to true
+ */
+function unloadExtension(uuid, type, deleteConfig = true) {
+    let extension = type.maps.objects[uuid];
     if (extension) {
         extension.unlockRole();
 
@@ -412,72 +495,109 @@ function unloadExtension(uuid) {
         // but it will be removed on next reboot, and hopefully nothing
         // broke too much.
         try {
-            extension.type.callbacks.prepareExtensionUnload(extension);
+            extension.type.callbacks.prepareExtensionUnload(extension, deleteConfig);
         } catch(e) {
             global.logError('Error disabling ' + extension.lowerType + ' ' + extension.uuid, e);
         }
         extension.unloadStylesheet();
+        extension.unloadIconDirectory();
 
         extension.type.emit('extension-unloaded', extension.uuid);
 
-        forgetExtension(extension.uuid, true);
+        forgetExtension(extension.uuid, type, true);
     }
 }
 
-function forgetExtension(uuid, forgetMeta) {
-    delete importObjects[uuid];
-    delete objects[uuid];
+function forgetExtension(uuid, type, forgetMeta) {
+    delete type.maps.importObjects[uuid];
+    delete type.maps.objects[uuid];
     if(forgetMeta)
-        delete meta[uuid];
+        delete type.maps.meta[uuid];
+}
+
+/**
+ * reloadExtension:
+ *
+ * @uuid (string): uuid of xlet
+ * @type (Extension.Type): type of xlet
+ *
+ * Reloads an xlet. Useful when the source has changed.
+ */
+function reloadExtension(uuid, type) {
+    let extension = type.maps.objects[uuid];
+
+    if(extension)
+        unloadExtension(uuid, type, false);
+
+    loadExtension(uuid, type);
 }
 
 function findExtensionDirectory(uuid, type) {
-    let directory = findExtensionDirectoryIn(uuid, type.userDir);
-    if (directory == null) {
-        let systemDataDirs = GLib.get_system_data_dirs();
-        for (let i = 0; i < systemDataDirs.length; i++) {
-            let dirPath = systemDataDirs[i] + '/cinnamon/' + type.folder;
-            let dir = Gio.file_new_for_path(dirPath);
-            if (dir.query_exists(null))
-                directory = findExtensionDirectoryIn(uuid, dir);
-                if (directory != null) {
-                    break;
-                }
-            }
+    let dirPath = type.userDir + "/" + uuid;
+    let dir = Gio.file_new_for_path(dirPath);
+    if (dir.query_file_type(Gio.FileQueryInfoFlags.NONE, null)
+            == Gio.FileType.DIRECTORY)
+        return dir;
+
+    let systemDataDirs = GLib.get_system_data_dirs();
+    for (let datadir of systemDataDirs) {
+        dirPath = datadir + '/cinnamon/' + type.folder + '/' + uuid;
+        dir = Gio.file_new_for_path(dirPath);
+        if (dir.query_file_type(Gio.FileQueryInfoFlags.NONE, null)
+                == Gio.FileType.DIRECTORY)
+            return dir;
     }
-    return directory;
+    return null;
 }
 
-function findExtensionDirectoryIn(uuid, dir) {
+/**
+ * findExtensionSubdirectory:
+ * @dir (Gio.File): directory to search in
+ *
+ * For extensions that are shipped with multiple versions in different
+ * directories, look for the largest available version that is less than or
+ * equal to the current running version. If no such version is found, the
+ * original directory is returned.
+ *
+ * Returns (Gio.File): directory object of the desired directory.
+ */
+function findExtensionSubdirectory(dir) {
     try {
         let fileEnum = dir.enumerate_children('standard::*', Gio.FileQueryInfoFlags.NONE, null);
 
-        let directory = null;
         let info;
+        let largest = null;
         while ((info = fileEnum.next_file(null)) != null) {
             let fileType = info.get_file_type();
             if (fileType != Gio.FileType.DIRECTORY)
                 continue;
+
             let name = info.get_name();
-            if (name == uuid) {
-                let child = dir.get_child(name);
-                directory = child;
-                break;
+            if (!name.match(/^[1-9][0-9]*\.[0-9]+(\.[0-9]+)?$/))
+                continue;
+
+            if (versionLeq(name, Config.PACKAGE_VERSION) &&
+                (!largest || versionLeq(largest[0], name))) {
+                largest = [name, fileEnum.get_child(info)];
             }
         }
 
         fileEnum.close(null);
-        return directory;
+        if (largest)
+            return largest[1];
+        else
+            return dir;
     } catch (e) {
-        global.logError('Error looking for extension ' + uuid + ' in directory ' + dir, e);
-       return null;
+        global.logError('Error looking for extension version for ' + dir.get_basename() + ' in directory ' + dir, e);
+        return dir;
     }
 }
 
-function get_max_instances (uuid) {
-    if (uuid in meta) {
-        if ("max-instances" in meta[uuid]) {
-            return parseInt(meta[uuid]["max-instances"]);
+function get_max_instances (uuid, type) {
+    if (uuid in type.maps.meta) {
+        if ("max-instances" in type.maps.meta[uuid]) {
+            let i = type.maps.meta[uuid]["max-instances"];
+            return parseInt(i);
         }
     }
     return 1;
