@@ -29,6 +29,8 @@ WindowAttentionHandler.prototype = {
             (window.get_wm_class() && (window.get_wm_class().indexOf("Skype") > -1 ||
                                        window.get_wm_class().indexOf("Quassel") > -1 ||
                                        window.get_wm_class().indexOf("Pidgin") > -1 ||
+                                       window.get_wm_class().indexOf("RetroShare") > -1 ||
+                                       window.get_wm_class().indexOf("Psi-plus") > -1 ||                                       
                                        window.get_wm_class().indexOf("Kadu") > -1)))
             return;
 
