@@ -23,6 +23,7 @@
 #include <string.h>
 
 #include "st-private.h"
+#include "st-cogl-wrapper.h"
 
 /**
  * _st_actor_get_preferred_width:
@@ -368,12 +369,11 @@ _st_create_texture_material (CoglHandle src_texture)
       static const guint8 white_pixel[] = { 0xff, 0xff, 0xff, 0xff };
       CoglHandle dummy_texture;
 
-      dummy_texture =
-        cogl_texture_new_from_data (1, 1,
-                                    COGL_TEXTURE_NONE,
-                                    COGL_PIXEL_FORMAT_RGBA_8888_PRE,
-                                    COGL_PIXEL_FORMAT_ANY,
-                                    4, white_pixel);
+      dummy_texture = st_cogl_texture_new_from_data_wrapper (1, 1,
+                                                             COGL_TEXTURE_NONE,
+                                                             COGL_PIXEL_FORMAT_RGBA_8888_PRE,
+                                                             COGL_PIXEL_FORMAT_ANY,
+                                                             4, white_pixel);
 
       texture_material_template = cogl_material_new ();
       cogl_material_set_layer (texture_material_template, 0, dummy_texture);
@@ -557,13 +557,12 @@ _st_create_shadow_material (StShadow   *shadow_spec,
                             &width_out, &height_out, &rowstride_out);
   g_free (pixels_in);
 
-  texture = cogl_texture_new_from_data (width_out,
-                                        height_out,
-                                        COGL_TEXTURE_NONE,
-                                        COGL_PIXEL_FORMAT_A_8,
-                                        COGL_PIXEL_FORMAT_A_8,
-                                        rowstride_out,
-                                        pixels_out);
+  texture = st_cogl_texture_new_from_data_wrapper (width_out, height_out,
+                                                   COGL_TEXTURE_NONE,
+                                                   COGL_PIXEL_FORMAT_A_8,
+                                                   COGL_PIXEL_FORMAT_A_8,
+                                                   rowstride_out,
+                                                   pixels_out);
 
   g_free (pixels_out);
 
@@ -614,10 +613,9 @@ _st_create_shadow_material_from_actor (StShadow     *shadow_spec,
       if (width == 0 || height == 0)
         return COGL_INVALID_HANDLE;
 
-      buffer = cogl_texture_new_with_size (width,
-                                           height,
-                                           COGL_TEXTURE_NO_SLICING,
-                                           COGL_PIXEL_FORMAT_ANY);
+      buffer = st_cogl_texture_new_with_size_wrapper (width, height,
+                                                      COGL_TEXTURE_NO_SLICING,
+                                                      COGL_PIXEL_FORMAT_ANY);
 
       if (buffer == COGL_INVALID_HANDLE)
         return COGL_INVALID_HANDLE;
