@@ -35,6 +35,7 @@
 #include "st-theme-context.h"
 #include "st-texture-cache.h"
 #include "st-theme-node-private.h"
+#include "st-cogl-wrapper.h"
 
 /****
  * Rounded corners
@@ -165,12 +166,13 @@ create_corner_material (StCornerSpec *corner)
 
   cairo_surface_destroy (surface);
 
-  texture = cogl_texture_new_from_data (size, size,
-                                        COGL_TEXTURE_NONE,
-                                        CLUTTER_CAIRO_FORMAT_ARGB32,
-                                        COGL_PIXEL_FORMAT_ANY,
-                                        rowstride,
-                                        data);
+  texture = st_cogl_texture_new_from_data_wrapper (size, size,
+                                                   COGL_TEXTURE_NONE,
+                                                   CLUTTER_CAIRO_FORMAT_ARGB32,
+                                                   COGL_PIXEL_FORMAT_ANY,
+                                                   rowstride,
+                                                   data);
+
   g_free (data);
   g_assert (texture != COGL_INVALID_HANDLE);
 
@@ -1249,13 +1251,13 @@ st_theme_node_prerender_background (StThemeNode *node)
   if (interior_path != NULL)
     cairo_path_destroy (interior_path);
 
-  texture = cogl_texture_new_from_data (paint_box.x2 - paint_box.x1,
-                                        paint_box.y2 - paint_box.y1,
-                                        COGL_TEXTURE_NONE,
-                                        CLUTTER_CAIRO_FORMAT_ARGB32,
-                                        COGL_PIXEL_FORMAT_ANY,
-                                        rowstride,
-                                        data);
+  texture = st_cogl_texture_new_from_data_wrapper (paint_box.x2 - paint_box.x1,
+                                                   paint_box.y2 - paint_box.y1,
+                                                   COGL_TEXTURE_NONE,
+                                                   CLUTTER_CAIRO_FORMAT_ARGB32,
+                                                   COGL_PIXEL_FORMAT_ANY,
+                                                   rowstride,
+                                                   data);
 
   cairo_destroy (cr);
   cairo_surface_destroy (surface);
@@ -1438,10 +1440,10 @@ st_theme_node_render_resources (StThemeNode   *node,
           int texture_width = ceil (width);
           int texture_height = ceil (height);
 
-          buffer = cogl_texture_new_with_size (texture_width,
-                                               texture_height,
-                                               COGL_TEXTURE_NO_SLICING,
-                                               COGL_PIXEL_FORMAT_ANY);
+          buffer = st_cogl_texture_new_with_size_wrapper (texture_width,
+                                                          texture_height,
+                                                          COGL_TEXTURE_NO_SLICING,
+                                                          COGL_PIXEL_FORMAT_ANY);
           offscreen = cogl_offscreen_new_to_texture (buffer);
 
           if (offscreen != COGL_INVALID_HANDLE)
